# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++


class ContabilidadController < BaseImportacionController

  before_filter :elementos_filtrado, :only => [ :listado ]
  before_filter :verificar_etapa, :only => [ :resumen ]

  def index
    redirect_to :action => "listado"
  end

  # en agentes: establece los parametros de ordenación
  def ordenado
    session[:contabilidad_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC"
    session[:contabilidad_orden] = params[:orden] ? params[:orden] : "codigo"
    redirect_to :action => params[:listado] || "listado"
  end

  def filtrado
    session[:contabilidad_filtro_tipo] = params[:filtro][:tipo] if params[:filtro]
    session[:contabilidad_filtro_centro_coste] = params[:filtro][:centro_coste] if params[:filtro]
    redirect_to :action => params[:listado] || "listado"
  end

  def elementos_filtrado
    session[:contabilidad_asc_desc] ||= "ASC"
    session[:contabilidad_orden] ||= "codigo"
    session[:contabilidad_filtro_tipo] ||= "todos"
    session[:contabilidad_filtro_centro_coste] ||= "todos"

    @tipo = session[:contabilidad_filtro_tipo] unless session[:contabilidad_filtro_tipo] == "todos"
    @centro_coste = session[:contabilidad_filtro_centro_coste] unless session[:contabilidad_filtro_centro_coste] == "todos"

    filtro_tipo = [[_("Todos"),"todos"]] + tipos_de_elemento_en_subcuentas
    filtro_centro_coste = [[_("Todos"),"todos"], [_("Sólo Subcuentas"), "false"], [_("Sólo Centros de Coste"), "true"]] 
    @opciones_filtrado = [{:rotulo =>  _("Elemento Contable"), :nombre => "centro_coste", :opciones => filtro_centro_coste},
                          {:rotulo =>  _("Tipo de subcuenta"), :nombre => "tipo", :opciones => filtro_tipo},
                         ]
    @accion_filtrado = {:action => :filtrado, :listado => :listado}
    @estado_filtrado = [ (@centro_coste ? (@centro_coste == "false" ? _("Sólo Subcuentas") : _("Sólo Centros de Coste")) : _("Subcuentas y Centros de Coste") ),
                         (@tipo || _("Todos los tipos")) ]
  end


	# en agentes: lista las cuentas contables y los centros de coste
  def listado 
    elementos_filtrado
    # Esto lo hacemos por posibles conflictos con el "nombre" del agente
    condiciones = {:agente_id => @agente.id}
    condiciones[:elemento_contable_type] = @tipo if @tipo
    condiciones[:centro_coste] = (@centro_coste == "true") if @centro_coste

    @subcuentas = @paginado = CuentaContable.paginate(:conditions => condiciones,
                :order => (session[:contabilidad_orden] + " " + session[:contabilidad_asc_desc] ),
                :page => (params[:format]=='xls' ? nil : params[:page]), :per_page => (params[:format_xls_count] || session[:por_pagina]) ) 
    
    @formato_xls = @subcuentas.total_entries
    respond_to do |format|
      format.html
      format.xls do
        @tipo = "subcuenta_contable"
        @objetos = @subcuentas
        nom_fich = "mapeo_contable_" + @agente.nombre + "_" + Time.now.strftime("%Y-%m-%d")
        render 'comunes_xls/listado', :xls => nom_fich, :layout => false
      end
    end
  end

	# en agentes: prepara el formulario de crear o editar
  def editar_nuevo
    @subcuenta = @agente.cuentas_contables_delegacion.find_by_id(params[:id]) || CuentaContable.new(:agente_id => @agente.id)
    @tipo_elementos = tipos_de_elemento_en_subcuentas
    @elementos = elementos_por_tipo_subcuenta @subcuenta.elemento_contable_type
    render (:update) { |page| page.formulario(:partial => "formulario", :update => params[:update]) }
  end

	# en agentes: modifica o crea una 
  def modificar_crear
    @subcuenta = @agente.cuentas_contables_delegacion.find_by_id(params[:id]) || CuentaContable.new(:agente_id => @agente.id)
    @subcuenta.update_attributes params[:subcuenta]
    if @subcuenta.errors.empty?
      # Si es una ya existente, modifica la linea
      render(:update) { |page|   page.modificar :update => params[:update], :partial => "subcuenta" , :mensaje => { :errors => @subcuenta.errors } } if params[:id]
      # Si es una nueva la incluye en la parte superior del listado
      render :update do |page|
        page.show "nuevas_subcuentas"
        page.modificar :update => "subcuenta_nueva_" + params[:i], :partial => "nueva_subcuenta", :mensaje => { :errors => @subcuenta.errors }
        page.replace "formulario_anadir_anadir", :inline => "<%= anadir(:url => {:action => 'editar_nuevo', :i => (params[:i].to_i + 1).to_s, :update => 'formulario_anadir'}) %>" unless params[:id]
      end unless params[:id] 
    else
      @tipo_elementos = tipos_de_elemento_en_subcuentas
      @elementos = elementos_por_tipo_subcuenta @subcuenta.elemento_contable_type
      render(:update) { |page| page.recargar_formulario :partial => "formulario", :mensaje => {:errors => @subcuenta.errors} }
    end 
  end

	# en agentes: elimina una subcuenta
  def eliminar
    @subcuenta = CuentaContable.find_by_id(params[:id])
    @subcuenta.destroy if @subcuenta
    render (:update) {|page| page.eliminar :update => params[:update], :mensaje => {:errors => @subcuenta.errors, :eliminar => true}}
  end

	# en agentes: importa las subcuentas, centros de coste o libro diario
  def importacion
    if params[:selector] && params[:selector][:tipo] && params[:selector][:fichero]
      begin
        tmp = params[:selector][:fichero].tempfile
        Spreadsheet.client_encoding = 'UTF-8'

        book = Spreadsheet.open tmp.path

        case params[:selector][:tipo]
          when "diario" then
            importa_diario book.worksheet(0)
          when "subcuenta" then
            importa_mapeo book.worksheet(0), false
          when "centro_coste" then
            importa_mapeo book.worksheet(0), true
        end
        @import_error ||= ""
      rescue Ole::Storage::FormatError
        importacion_error _("ERROR: El documento no es una hoja de cálculo válida")
      rescue => ex
        logger.error ex.message
        logger.error ex.backtrace
        importacion_error _("Error inesperado: Contacte con el administrador del sistema.")
      end
    end
  end


  #-- 
  # METODOS AJAX PARA CAMBIO DE TIPO 
  # ++

  def subcuenta_cambio_tipo
    @elementos = elementos_por_tipo_subcuenta params[:type]
    render :partial => "elemento"
  end

 private
  # Comprueba que exista etapa si estamos en agentes
  def verificar_etapa
    if @agente && @agente.etapa.empty?
      msg_error _("Tiene que definir por lo menos una etapa para acceder a la gestión de Subcuentas Contables")
      redirect_to :menu => :configuracion_agente, :controller => :datos_agente, :action => :etapas
    end
  end

  # Obtiene los tipos de elementos de subcuentas para el filtro del listado
  def tipos_de_elemento_en_subcuentas
    [ [_("Proyecto"), "Proyecto"], [_("Financiador"), "Agente"], [_("Cuenta/Caja"), "Libro"], [_("Partida"), "Partida"] ] 
  end

  def elementos_por_tipo_subcuenta tipo_elemento
    objs = nil
    # Lo hacemos en un case y no en un eval porque cada tipo de elemento tiene sus cosillas
    case tipo_elemento
      when "Proyecto"
        # Evitamos mapeo contra convenios
        objs = Proyecto.where(:convenio_accion => nil).order(:nombre).collect{|p| [p.nombre, p.id] }
      when "Libro"
        objs = @agente.libro.order(:nombre).collect{|p| [p.nombre, p.id] }
      when "Agente"
        objs = Agente.where(:financiador => true).order(:nombre).collect{|p| [p.nombre, p.id] }
      when "Partida"
        objs = Partida.order(:codigo).collect{|p| [p.codigo_nombre, p.id] }
    end
    return objs
  end

  def importa_mapeo hoja, centro_coste=false
    # La estructura de subcuentas de una hoja es:
    columnas = [ "codigo", "descripcion" ]
    indice = Hash[*columnas.each_with_index.to_a.flatten]

    CuentaContable.transaction do
      # Elimina todas las cuentas o centros de coste que no esten vinculadas aun
      CuentaContable.where(:agente_id => @agente.id, :elemento_contable_id => nil, :centro_coste => centro_coste).each { |cc| cc.destroy }
      # Itera cada fila 
      hoja.each_with_index do |fila,numero_fila|
        (tipo, codigo) = tipo_elemento_por_codigo_subcuenta fila[indice['codigo']].to_s unless centro_coste
        (tipo, codigo) = tipo_elemento_por_codigo_centro_coste fila[indice['codigo']].to_s if centro_coste
        if tipo
          cc = CuentaContable.find_or_create_by_codigo_and_agente_id_and_centro_coste codigo, @agente.id, centro_coste
          if cc.elemento_contable_id.nil? || cc.elemento_contable_type == tipo
            cc.elemento_contable_type = tipo
            cc.descripcion = fila[indice['descripcion']]
            cc.save
            importacion_error cc
            importacion_error _("Error procesando la línea %{num}.")%{:num => numero_fila} unless cc.errors.empty?
          else
            importacion_error _("Se está modificando el tipo de una 'subcuenta contable' ya vinculada") unless centro_coste
            importacion_error _("Se está modificando el tipo de un 'centro de coste' ya vinculado") if centro_coste
          end
        end
      end     
    end 
  end

  def importa_diario hoja
    # La estructura de subcuentas de una hoja es:
    columnas = [ "apunte", "asiento", "fecha", "referencia", "apunte_2", "partida", "nom_partida", "contrapartida", "concepto", "clasifica", "debe", "haber", "analitica", "rdos" ]
    indice = Hash[*columnas.each_with_index.to_a.flatten]

    diario = {}
    CuentaContable.transaction do
      # Itera cada fila 
      hoja.each_with_index do |fila,numero_fila|
        limpia_fila fila

        apunte = fila[indice['partida']].to_i
        asiento = fila[indice['asiento']].to_i
        diario[apunte] ||= {}
        diario[apunte][asiento] = fila

        cuenta_orig = fila[indice['partida']].to_s
        cuenta_dest = fila[indice['contrapartida']].to_s

        elemento_orig = codigo_contable_por_subcuenta cuenta_orig
        elemento_dest = codigo_contable_por_subcuenta cuenta_dest

        # Subvencion recibida
        if cuenta_orig.match(/^57/) && cuenta_dest.match(/^4708/)
        # Intereses
        elsif cuenta_orig.match(/^57/) && cuenta_dest.match(/^522/)
          libro = elemento_orig if elemento_orig.class.name == "Libro"
          proyecto = elemento_dest if elemento_dest.class.name == "Proyecto"
          if libro && proyecto
            # Genera una transferencia de tipo "interes"
          else
            importacion_error _("Fila %{num}: No se pudo encontrar el libro con codigo contable %{cod}")%{:num => numero_fila, :cod => cuenta_orig} unless libro
            importacion_error _("Fila %{num}: No se pudo encontrar el proyecto con codigo contable %{cod}")%{:num => numero_fila, :cod => cuenta_dest} unless proyecto 
          end 
        # Transferencia (aportaciones)
        elsif cuenta_orig.match(/^651/) && cuenta_dest.match(/^57/)
          # A rellenar el codigo de aqui
        # Gastos indirectos
        elsif cuenta_orig.match(/^74/) && cuenta_dest.match(/^522/)
          # A rellenar el codigo de aqui
        # Gastos financieros
        elsif cuenta_orig.match(/^6260000/) && cuenta_dest.match(/^57/)
          # A rellenar el codigo de aqui
        # Gastos con pagos asociados
        elsif cuenta_orig.match(/^6/) && cuenta_dest.match(/^57/)
          partida = elemento_orig if elemento_dest.class.name == "Partida"
          libro = elemento_dest if elemento_dest.class.name == "Libro"
          if partida && libro
            agente = libro.agente
            # El pais seria el pais de origen del agente?
          else
            importacion_error _("Fila %{num}: No se pudo encontrar el partida con codigo contable %{cod}")%{:num => numero_fila, :cod => cuenta_orig} unless partida
            importacion_error _("Fila %{num}: No se pudo encontrar el libro con codigo contable %{cod}")%{:num => numero_fila, :cod => cuenta_dest} unless libro
          end
        end
      end
    end
  end

  # Obtiene el tipo de elemento segun el codigo del centro de coste
  # Este mapeo hay que codificarlo de otra forma pues depende de como se haya estructurado la analitica de la organizacion
  def tipo_elemento_por_codigo_centro_coste codigo
    tipo = nil
    codigo_retocado = codigo
    # Gastos de Actividad
    if codigo.match(/^GAP(?<proy>\d{3})$/)
      tipo = "Proyecto"
      #codigo_retocado = "CC#P" + $+ + "x"
      codigo_retocado = "522" + $+ + "x_ap"
    elsif codigo.match(/^GAC(?<proy>\d{3})$/)
      tipo = "Proyecto"
      #codigo_retocado = "CC#C" + $+ + "x"
      codigo_retocado = "522" + $+ + "x_ac"
    # Gastos Estructura
    elsif codigo.match(/^GEPP(?<proy>\d{3})$/)
      tipo = "Proyecto"
      #codigo_retocado = "CC#P" + $+ + "x"
      codigo_retocado = "522" + $+ + "x_ep"
    elsif codigo.match(/^GEPC(?<proy>\d{3})$/)
      tipo = "Proyecto"
      #codigo_retocado = "CC#C" + $+ + "x"
      codigo_retocado = "522" + $+ + "x_ec"
    else
      tipo = "_Otro_"
    end
    # Hemos usado el codigo retocado "CC#" pues los codigos de proyecto en actividad y en estructura son iguales
    return tipo, codigo_retocado
  end

  # Devuelve el elemento con un codigo de subcuenta
  def codigo_contable_por_subcuenta subcuenta
    (tipo, codigo) = tipo_elemento_por_codigo_subcuenta subcuenta
    cc = CuentaContable.find_by_agente_id_and_elemento_contable_type_and_codigo @agente.id, tipo, codigo
    return cc ? cc.elemento_contable : nil
  end

  # Obtiene el tipo de elemento segun el codigo de cuenta
  # Este mapeo hay que codificarlo de otra forma. Probablemente solo corresponda a la contabilidad española
  def tipo_elemento_por_codigo_subcuenta codigo
    tipo = nil
    codigo_retocado = codigo
    # Subvencion
    if codigo.match(/^130(?<proy>\d{3})\d{1}/)
      tipo = "_Proyecto_"
      codigo_retocado = "xxx" + $+ + "x" + "-g130"
    # Donaciones 
    elsif codigo.match(/^131/)
      tipo = "_Donaciones_"
    # Subvencion
    elsif codigo.match(/^172(?<proy>\d{3})\d{1}/)
      tipo = "_Proyecto_"
      codigo_retocado = "xxx" + $+ + "x" + "-g172"
    elsif codigo.match(/^41/)
      tipo = "_Proveedor_"
    elsif codigo.match(/^43/)
      tipo = "_Cliente_"
    elsif codigo.match(/^44/)
      tipo = "_Financiador?_"
    elsif codigo.match(/^46/)
      tipo = "_Personal_"
    elsif codigo.match(/^4708/)
      tipo = "Agente"
    elsif codigo.match(/^522(?<proy>\d{3})\d{1}/)
      tipo = "Proyecto"
      codigo_retocado = "522" + $+ + "x"
    elsif codigo.match(/^552/)
      tipo = "_Deudas_a_corto_"
    # Libros
    elsif codigo.match(/^57/)
      tipo = "Libro"
    # Gasto
    elsif codigo.match(/^6/)
      tipo = "Partida"
    # Ingresos
    elsif codigo.match(/^74/)
      tipo = "_Ingresos_"
    # El resto, de momento lo metemos en otros
    else
      tipo = "_Otro_"
    end
    return tipo, codigo_retocado
  end
end

